@@ -40,20 +40,17 @@
 #include <glib/gi18n.h>
 #include <dbus/dbus-glib.h>
 #include <gconf/gconf-client.h>
+#include <devkit-power-gobject/devicekit-power.h>
 
-#include <hal-manager.h>
-
-#include "gpm-ac-adapter.h"
 #include "gpm-button.h"
 #include "gpm-backlight.h"
 #include "gpm-brightness.h"
 #include "gpm-control.h"
 #include "gpm-common.h"
 #include "egg-debug.h"
-#include "gpm-feedback-widget.h"
+#include "gpm-popup-window.h"
 #include "gpm-dpms.h"
 #include "gpm-idle.h"
-#include "gpm-light-sensor.h"
 #include "gpm-marshal.h"
 #include "gpm-stock-icons.h"
 #include "gpm-prefs-server.h"
@@ -62,27 +59,22 @@
 
 struct GpmBacklightPrivate
 {
-	GpmAcAdapter		*ac_adapter;
+	DkpClient		*client;
 	GpmBrightness		*brightness;
 	GpmButton		*button;
 	GConfClient		*conf;
-	GpmFeedback		*feedback;
+	GtkWidget		*popup;
 	GpmControl		*control;
 	GpmDpms			*dpms;
 	GpmIdle			*idle;
-	GpmLightSensor		*light_sensor;
 	gboolean		 can_dim;
-	gboolean		 can_sense;
-	gboolean		 is_laptop;
 	gboolean		 system_is_idle;
 	GTimer			*idle_timer;
-	gfloat			 ambient_sensor_value;
 	guint			 idle_dim_timeout;
 	guint			 master_percentage;
 };
 
 enum {
-	MODE_CHANGED,
 	BRIGHTNESS_CHANGED,
 	LAST_SIGNAL
 };
@@ -104,39 +96,6 @@ gpm_backlight_error_quark (void)
 	return quark;
 }
 
-/* dbus methods shouldn't use enumerated types, but should use textual descriptors */
-gboolean
-gpm_backlight_set_mode (GpmBacklight *backlight, const gchar *mode_str, GError **error)
-{
-	gboolean ret;
-	GpmDpmsMode mode;
-
-	g_return_val_if_fail (GPM_IS_BACKLIGHT (backlight), FALSE);
-
-	/* convert mode to an enumerated type */
-	mode = gpm_dpms_mode_from_string (mode_str);
-	ret = gpm_dpms_set_mode (backlight->priv->dpms, mode, error);
-	return ret;
-}
-
-/* dbus methods shouldn't use enumerated types, but should use textual descriptors */
-gboolean
-gpm_backlight_get_mode (GpmBacklight *backlight,
-			const gchar **mode_str,
-			GError      **error)
-{
-	gboolean ret;
-	GpmDpmsMode mode;
-
-	g_return_val_if_fail (GPM_IS_BACKLIGHT (backlight), FALSE);
-	g_return_val_if_fail (mode_str != NULL, FALSE);
-
-	ret = gpm_dpms_get_mode (backlight->priv->dpms, &mode, error);
-	if (ret)
-		*mode_str = g_strdup (gpm_dpms_mode_to_string (mode));
-	return ret;
-}
-
 /**
  * gpm_backlight_get_brightness:
  **/
@@ -229,7 +188,7 @@ gpm_backlight_brightness_evaluate_and_set (GpmBacklight *backlight, gboolean int
 	gfloat brightness;
 	gfloat scale;
 	gboolean ret;
-	gboolean on_ac;
+	gboolean on_battery;
 	gboolean do_laptop_lcd;
 	gboolean enable_action;
 	gboolean battery_reduce;
@@ -252,12 +211,14 @@ gpm_backlight_brightness_evaluate_and_set (GpmBacklight *backlight, gboolean int
 	brightness = backlight->priv->master_percentage / 100.0f;
 	egg_debug ("1. main brightness %f", brightness);
 
-	/* get AC status */
-	on_ac = gpm_ac_adapter_is_present (backlight->priv->ac_adapter);
+	/* get battery status */
+	g_object_get (backlight->priv->client,
+		      "on-battery", &on_battery,
+		      NULL);
 
 	/* reduce if on battery power if we should */
 	battery_reduce = gconf_client_get_bool (backlight->priv->conf, GPM_CONF_BACKLIGHT_BATTERY_REDUCE, NULL);
-	if (on_ac == FALSE && battery_reduce) {
+	if (on_battery && battery_reduce) {
 		value = gconf_client_get_int (backlight->priv->conf, GPM_CONF_BACKLIGHT_BRIGHTNESS_DIM_BATT, NULL);
 		if (value > 100) {
 			egg_warning ("cannot use battery brightness value %i, correcting to 50", value);
@@ -271,7 +232,7 @@ gpm_backlight_brightness_evaluate_and_set (GpmBacklight *backlight, gboolean int
 	egg_debug ("2. battery scale %f, brightness %f", scale, brightness);
 
 	/* reduce if system is momentarily idle */
-	if (on_ac)
+	if (!on_battery)
 		enable_action = gconf_client_get_bool (backlight->priv->conf, GPM_CONF_BACKLIGHT_IDLE_DIM_AC, NULL);
 	else
 		enable_action = gconf_client_get_bool (backlight->priv->conf, GPM_CONF_BACKLIGHT_IDLE_DIM_BATT, NULL);
@@ -288,26 +249,6 @@ gpm_backlight_brightness_evaluate_and_set (GpmBacklight *backlight, gboolean int
 	}
 	egg_debug ("3. idle scale %f, brightness %f", scale, brightness);
 
-	/* reduce if ambient is low */
-	enable_action = gconf_client_get_bool (backlight->priv->conf, GPM_CONF_AMBIENT_ENABLE, NULL);
-	if (backlight->priv->can_sense && enable_action) {
-		value = gconf_client_get_int (backlight->priv->conf, GPM_CONF_AMBIENT_SCALE, NULL);
-		scale = backlight->priv->ambient_sensor_value * (value / 100.0f);
-		value = gconf_client_get_int (backlight->priv->conf, GPM_CONF_AMBIENT_FACTOR, NULL);
-		scale = gpm_common_sum_scale (brightness, scale, value / 100.0f);
-		if (scale > 1.0f) {
-			scale = 1.0f;
-		}
-		if (scale < 0.80f) {
-			brightness *= scale;
-		} else {
-			scale = 1.0f;
-		}
-	} else {
-		scale = 1.0f;
-	}
-	egg_debug ("4. ambient scale %f, brightness %f", scale, brightness);
-
 	/* convert to percentage */
 	value = (guint) ((brightness * 100.0f) + 0.5);
 
@@ -319,8 +260,10 @@ gpm_backlight_brightness_evaluate_and_set (GpmBacklight *backlight, gboolean int
 	}
 
 	/* only show dialog if interactive */
-	if (interactive)
-		gpm_feedback_display_value (backlight->priv->feedback, (float) brightness);
+	if (interactive) {
+		gpm_popup_window_set_value (GPM_POPUP_WINDOW (backlight->priv->popup), (float) brightness);
+		gtk_widget_show (backlight->priv->popup);
+	}
 
 	ret = gpm_brightness_set (backlight->priv->brightness, value, &hw_changed);
 	/* we emit a signal for the brightness applet */
@@ -340,25 +283,25 @@ static void
 gpm_conf_gconf_key_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, GpmBacklight *backlight)
 {
 	GConfValue *value;
-	gboolean on_ac;
+	gboolean on_battery;
 
 	value = gconf_entry_get_value (entry);
 	if (value == NULL)
 		return;
 
-	on_ac = gpm_ac_adapter_is_present (backlight->priv->ac_adapter);
+	/* get battery status */
+	g_object_get (backlight->priv->client,
+		      "on-battery", &on_battery,
+		      NULL);
 
-	if (on_ac && strcmp (entry->key, GPM_CONF_BACKLIGHT_BRIGHTNESS_AC) == 0) {
+	if (!on_battery && strcmp (entry->key, GPM_CONF_BACKLIGHT_BRIGHTNESS_AC) == 0) {
 		backlight->priv->master_percentage = gconf_value_get_int (value);
 		gpm_backlight_brightness_evaluate_and_set (backlight, FALSE);
 
-	} else if (!on_ac && strcmp (entry->key, GPM_CONF_BACKLIGHT_BRIGHTNESS_DIM_BATT) == 0) {
+	} else if (on_battery && strcmp (entry->key, GPM_CONF_BACKLIGHT_BRIGHTNESS_DIM_BATT) == 0) {
 		gpm_backlight_brightness_evaluate_and_set (backlight, FALSE);
 
 	} else if (strcmp (entry->key, GPM_CONF_BACKLIGHT_IDLE_DIM_AC) == 0 ||
-	         strcmp (entry->key, GPM_CONF_AMBIENT_ENABLE) == 0 ||
-	         strcmp (entry->key, GPM_CONF_AMBIENT_FACTOR) == 0 ||
-	         strcmp (entry->key, GPM_CONF_AMBIENT_SCALE) == 0 ||
 	         strcmp (entry->key, GPM_CONF_BACKLIGHT_ENABLE) == 0 ||
 	         strcmp (entry->key, GPM_CONF_TIMEOUT_SLEEP_DISPLAY_BATT) == 0 ||
 	         strcmp (entry->key, GPM_CONF_BACKLIGHT_BATTERY_REDUCE) == 0 ||
@@ -374,15 +317,14 @@ gpm_conf_gconf_key_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *e
 }
 
 /**
- * ac_adapter_changed_cb:
- * @ac_adapter: The ac_adapter class instance
- * @on_ac: if we are on AC power
- * @brightness: This class instance
+ * gpm_backlight_client_changed_cb:
+ * @client: The dkp_client class instance
+ * @backlight: This class instance
  *
  * Does the actions when the ac power source is inserted/removed.
  **/
 static void
-ac_adapter_changed_cb (GpmAcAdapter *ac_adapter, gboolean on_ac, GpmBacklight *backlight)
+gpm_backlight_client_changed_cb (DkpClient *client, GpmBacklight *backlight)
 {
 	gpm_backlight_brightness_evaluate_and_set (backlight, TRUE);
 }
@@ -410,7 +352,8 @@ gpm_backlight_button_pressed_cb (GpmButton *button, const gchar *type, GpmBackli
 		/* show the new value */
 		if (ret) {
 			gpm_brightness_get (backlight->priv->brightness, &percentage);
-			gpm_feedback_display_value (backlight->priv->feedback, (float) percentage/100.0f);
+			gpm_popup_window_set_value (GPM_POPUP_WINDOW (backlight->priv->popup), (gfloat) percentage/100.0f);
+			gtk_widget_show (backlight->priv->popup);
 			/* save the new percentage */
 			backlight->priv->master_percentage = percentage;
 		}
@@ -426,7 +369,8 @@ gpm_backlight_button_pressed_cb (GpmButton *button, const gchar *type, GpmBackli
 		/* show the new value */
 		if (ret) {
 			gpm_brightness_get (backlight->priv->brightness, &percentage);
-			gpm_feedback_display_value (backlight->priv->feedback, (float) percentage/100.0f);
+			gpm_popup_window_set_value (GPM_POPUP_WINDOW (backlight->priv->popup), (gfloat) percentage/100.0f);
+			gtk_widget_show (backlight->priv->popup);
 			/* save the new percentage */
 			backlight->priv->master_percentage = percentage;
 		}
@@ -513,7 +457,7 @@ idle_changed_cb (GpmIdle *idle, GpmIdleMode mode, GpmBacklight *backlight)
 {
 	gboolean ret;
 	GError *error = NULL;
-	gboolean on_ac;
+	gboolean on_battery;
 	gchar *dpms_method;
 	GpmDpmsMode dpms_mode;
 
@@ -553,8 +497,10 @@ idle_changed_cb (GpmIdle *idle, GpmIdleMode mode, GpmBacklight *backlight)
 		gpm_backlight_brightness_evaluate_and_set (backlight, FALSE);
 
 		/* get the DPMS state we're supposed to use on the power state */
-		on_ac = gpm_ac_adapter_is_present (backlight->priv->ac_adapter);
-		if (on_ac)
+		g_object_get (backlight->priv->client,
+			      "on-battery", &on_battery,
+			      NULL);
+		if (!on_battery)
 			dpms_method = gconf_client_get_string (backlight->priv->conf, GPM_CONF_BACKLIGHT_DPMS_METHOD_AC, NULL);
 		else
 			dpms_method = gconf_client_get_string (backlight->priv->conf, GPM_CONF_BACKLIGHT_DPMS_METHOD_BATT, NULL);
@@ -579,20 +525,6 @@ idle_changed_cb (GpmIdle *idle, GpmIdleMode mode, GpmBacklight *backlight)
 	}
 }
 
-/**
- * mode_changed_cb:
- * @mode: The DPMS mode, e.g. GPM_DPMS_MODE_OFF
- * @manager: This class instance
- *
- * What happens when the DPMS mode is changed.
- **/
-static void
-mode_changed_cb (GpmDpms *dpms, GpmDpmsMode mode, GpmBacklight *backlight)
-{
-	egg_debug ("emitting mode-changed : %s", gpm_dpms_mode_to_string (mode));
-	g_signal_emit (backlight, signals [MODE_CHANGED], 0, gpm_dpms_mode_to_string (mode));
-}
-
 /**
  * brightness_changed_cb:
  * @brightness: The GpmBrightness class instance
@@ -604,10 +536,6 @@ mode_changed_cb (GpmDpms *dpms, GpmDpmsMode mode, GpmBacklight *backlight)
 static void
 brightness_changed_cb (GpmBrightness *brightness, guint percentage, GpmBacklight *backlight)
 {
-	/* display the widget when something else changed the backlight */
-	egg_debug ("Need to display backlight feedback value %i", percentage);
-	gpm_feedback_display_value (backlight->priv->feedback, (float) percentage / 100.0f);
-
 	/* save the new percentage */
 	backlight->priv->master_percentage = percentage;
 
@@ -616,22 +544,6 @@ brightness_changed_cb (GpmBrightness *brightness, guint percentage, GpmBacklight
 	g_signal_emit (backlight, signals [BRIGHTNESS_CHANGED], 0, percentage);
 }
 
-/**
- * brightness_changed_cb:
- * @brightness: The GpmBrightness class instance
- * @percentage: The new percentage brightness
- * @brightness: This class instance
- *
- * This callback is called when the brightness value changes.
- **/
-static void
-sensor_changed_cb (GpmLightSensor *sensor, guint percentage, GpmBacklight *backlight)
-{
-	egg_debug ("sensor changed! %i", percentage);
-	backlight->priv->ambient_sensor_value = percentage / 100.0f;
-	gpm_backlight_brightness_evaluate_and_set (backlight, FALSE);
-}
-
 /**
  * control_resume_cb:
  * @control: The control class instance
@@ -665,13 +577,12 @@ gpm_backlight_finalize (GObject *object)
 	backlight = GPM_BACKLIGHT (object);
 
 	g_timer_destroy (backlight->priv->idle_timer);
+	gtk_widget_destroy (backlight->priv->popup);
 
 	g_object_unref (backlight->priv->dpms);
 	g_object_unref (backlight->priv->control);
-	g_object_unref (backlight->priv->light_sensor);
-	g_object_unref (backlight->priv->feedback);
 	g_object_unref (backlight->priv->conf);
-	g_object_unref (backlight->priv->ac_adapter);
+	g_object_unref (backlight->priv->client);
 	g_object_unref (backlight->priv->button);
 	g_object_unref (backlight->priv->idle);
 	g_object_unref (backlight->priv->brightness);
@@ -697,14 +608,6 @@ gpm_backlight_class_init (GpmBacklightClass *klass)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__UINT,
 			      G_TYPE_NONE, 1, G_TYPE_UINT);
-	signals [MODE_CHANGED] =
-		g_signal_new ("mode-changed",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GpmBacklightClass, mode_changed),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__STRING,
-			      G_TYPE_NONE, 1, G_TYPE_STRING);
 
 	g_type_class_add_private (klass, sizeof (GpmBacklightPrivate));
 }
@@ -720,8 +623,7 @@ gpm_backlight_class_init (GpmBacklightClass *klass)
 static void
 gpm_backlight_init (GpmBacklight *backlight)
 {
-	HalManager *hal_manager;
-	guint value;
+	gboolean lid_is_present = TRUE;
 	GpmPrefsServer *prefs_server;
 
 	backlight->priv = GPM_BACKLIGHT_GET_PRIVATE (backlight);
@@ -729,40 +631,39 @@ gpm_backlight_init (GpmBacklight *backlight)
 	/* record our idle time */
 	backlight->priv->idle_timer = g_timer_new ();
 
-	/* this has a delay.. */
-	backlight->priv->light_sensor = gpm_light_sensor_new ();
-	g_signal_connect (backlight->priv->light_sensor, "sensor-changed",
-			  G_CALLBACK (sensor_changed_cb), backlight);
-
-	/* watch for manual brightness changes (for the feedback widget) */
+	/* watch for manual brightness changes (for the popup widget) */
 	backlight->priv->brightness = gpm_brightness_new ();
 	g_signal_connect (backlight->priv->brightness, "brightness-changed",
 			  G_CALLBACK (brightness_changed_cb), backlight);
 
+	/* we use dkp_client for the ac-adapter-changed signal */
+	backlight->priv->client = dkp_client_new ();
+	g_signal_connect (backlight->priv->client, "changed",
+			  G_CALLBACK (gpm_backlight_client_changed_cb), backlight);
+
 	/* gets caps */
 	backlight->priv->can_dim = gpm_brightness_has_hw (backlight->priv->brightness);
-	backlight->priv->can_sense = gpm_light_sensor_has_hw (backlight->priv->light_sensor);
 
-	/* we use hal to see if we are a laptop */
-	hal_manager = hal_manager_new ();
-	backlight->priv->is_laptop = hal_manager_is_laptop (hal_manager);
-	g_object_unref (hal_manager);
+	/* we use DeviceKit-power to see if we should show the lid UI */
+#if DKP_CHECK_VERSION(0x009)
+	g_object_get (backlight->priv->client,
+		      "lid-is-present", &lid_is_present,
+		      NULL);
+#endif
 
 	/* expose ui in prefs program */
 	prefs_server = gpm_prefs_server_new ();
-	if (backlight->priv->is_laptop)
+	if (lid_is_present)
 		gpm_prefs_server_set_capability (prefs_server, GPM_PREFS_SERVER_LID);
 	if (backlight->priv->can_dim)
 		gpm_prefs_server_set_capability (prefs_server, GPM_PREFS_SERVER_BACKLIGHT);
-	if (backlight->priv->can_sense)
-		gpm_prefs_server_set_capability (prefs_server, GPM_PREFS_SERVER_AMBIENT);
 	g_object_unref (prefs_server);
 
 	/* watch for dim value changes */
 	backlight->priv->conf = gconf_client_get_default ();
 
 	/* watch gnome-power-manager keys */
-	gconf_client_add_dir (backlight->priv->conf, GPM_CONF_DIR, GCONF_CLIENT_PRELOAD_NONE, NULL);
+	gconf_client_add_dir (backlight->priv->conf, GPM_CONF_DIR, GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
 	gconf_client_notify_add (backlight->priv->conf, GPM_CONF_DIR,
 				 (GConfClientNotifyFunc) gpm_conf_gconf_key_changed_cb,
 				 backlight, NULL, NULL);
@@ -776,11 +677,6 @@ gpm_backlight_init (GpmBacklight *backlight)
 	g_signal_connect (backlight->priv->button, "button-pressed",
 			  G_CALLBACK (gpm_backlight_button_pressed_cb), backlight);
 
-	/* we use ac_adapter for the ac-adapter-changed signal */
-	backlight->priv->ac_adapter = gpm_ac_adapter_new ();
-	g_signal_connect (backlight->priv->ac_adapter, "ac-adapter-changed",
-			  G_CALLBACK (ac_adapter_changed_cb), backlight);
-
 	/* watch for idle mode changes */
 	backlight->priv->idle = gpm_idle_new ();
 	g_signal_connect (backlight->priv->idle, "idle-changed",
@@ -792,14 +688,13 @@ gpm_backlight_init (GpmBacklight *backlight)
 	gpm_idle_set_timeout_dim (backlight->priv->idle, backlight->priv->idle_dim_timeout);
 
 	/* use a visual widget */
-	backlight->priv->feedback = gpm_feedback_new ();
-	gpm_feedback_set_icon_name (backlight->priv->feedback,
-				    GPM_STOCK_BRIGHTNESS_LCD);
+	backlight->priv->popup = gpm_popup_window_new ();
+        gtk_window_set_position (GTK_WINDOW (backlight->priv->popup), GTK_WIN_POS_NONE);
+	gpm_popup_window_set_icon_name (GPM_POPUP_WINDOW (backlight->priv->popup),
+					GPM_STOCK_BRIGHTNESS_LCD);
 
 	/* DPMS mode poll class */
 	backlight->priv->dpms = gpm_dpms_new ();
-	g_signal_connect (backlight->priv->dpms, "mode-changed",
-			  G_CALLBACK (mode_changed_cb), backlight);
 
 	/* we refresh DPMS on resume */
 	backlight->priv->control = gpm_control_new ();
@@ -807,8 +702,6 @@ gpm_backlight_init (GpmBacklight *backlight)
 			  G_CALLBACK (control_resume_cb), backlight);
 
 	/* sync at startup */
-	gpm_light_sensor_get_absolute (backlight->priv->light_sensor, &value);
-	backlight->priv->ambient_sensor_value = value / 100.0f;
 	gpm_backlight_brightness_evaluate_and_set (backlight, FALSE);
 }
 
