@@ -2,7 +2,7 @@
  *
  * Copyright (C) 2005 Jaap Haitsma <jaap@haitsma.org>
  * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
- * Copyright (C) 2005-2007 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2005-2009 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -35,17 +35,12 @@
 
 #include "gpm-tray-icon.h"
 #include "gpm-common.h"
-#include "gpm-prefs.h"
 #include "gpm-prefs-core.h"
 #include "egg-debug.h"
 #include "gpm-stock-icons.h"
 #include "gpm-prefs-server.h"
 
-#ifdef HAVE_GCONF_DEFAULTS
-#include <polkit-gnome/polkit-gnome.h>
-#endif
-
-static void     gpm_prefs_finalize   (GObject	    *object);
+static void gpm_prefs_finalize (GObject *object);
 
 #define GPM_PREFS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_PREFS, GpmPrefsPrivate))
 
@@ -63,9 +58,6 @@ struct GpmPrefsPrivate
 	gboolean		 can_hibernate;
 	guint			 idle_delay;
 	GConfClient		*conf;
-#ifdef HAVE_GCONF_DEFAULTS
-	PolKitGnomeAction	*default_action;
-#endif
 };
 
 enum {
@@ -74,24 +66,10 @@ enum {
 	LAST_SIGNAL
 };
 
-static guint	     signals [LAST_SIGNAL] = { 0 };
+static guint signals [LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (GpmPrefs, gpm_prefs, G_TYPE_OBJECT)
 
-/* The text that should appear in the action combo boxes */
-#define ACTION_INTERACTIVE_TEXT		_("Ask me")
-#define ACTION_SUSPEND_TEXT		_("Suspend")
-#define ACTION_SHUTDOWN_TEXT		_("Shutdown")
-#define ACTION_HIBERNATE_TEXT		_("Hibernate")
-#define ACTION_BLANK_TEXT		_("Blank screen")
-#define ACTION_NOTHING_TEXT		_("Do nothing")
-
-/* If sleep time in a slider is set to 122 it is considered as never.
- * We use 122 because gnome-screensaver's idle delay can be pushed to 120.
- * We leave ourselves an extra minute past that (because our slider is always
- * at least idle_delay+1).  Then 122 is the special value 'never'. */
-const int NEVER_TIME_ON_SLIDER = 122;
-
 /**
  * gpm_prefs_class_init:
  * @klass: This prefs class instance
@@ -183,8 +161,7 @@ out:
  * @prefs: This prefs class instance
  **/
 static void
-gpm_prefs_help_cb (GtkWidget *widget,
-		   GpmPrefs  *prefs)
+gpm_prefs_help_cb (GtkWidget *widget, GpmPrefs *prefs)
 {
 	egg_debug ("emitting action-help");
 	g_signal_emit (prefs, signals [ACTION_HELP], 0);
@@ -195,14 +172,13 @@ gpm_prefs_help_cb (GtkWidget *widget,
  * @widget: The GtkWidget object
  **/
 static void
-gpm_prefs_icon_radio_cb (GtkWidget *widget,
-			 GpmPrefs  *prefs)
+gpm_prefs_icon_radio_cb (GtkWidget *widget, GpmPrefs *prefs)
 {
 	const gchar *str;
 	gint policy;
 
 	policy = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "policy"));
-	str = gpm_tray_icon_mode_to_string (policy);
+	str = gpm_icon_policy_to_string (policy);
 	egg_debug ("Changing %s to %s", GPM_CONF_UI_ICON_POLICY, str);
 	gconf_client_set_string (prefs->priv->conf, GPM_CONF_UI_ICON_POLICY, str, NULL);
 }
@@ -213,116 +189,11 @@ gpm_prefs_icon_radio_cb (GtkWidget *widget,
  * @value: The value in %.
  **/
 static gchar *
-gpm_prefs_format_percentage_cb (GtkScale *scale,
-				gdouble   value)
+gpm_prefs_format_percentage_cb (GtkScale *scale, gdouble value)
 {
 	return g_strdup_printf ("%.0f%%", value);
 }
 
-/**
- * gpm_prefs_format_time_cb:
- * @scale: The GtkScale object
- * @value: The value in minutes.
- * @prefs: This prefs class instance
- **/
-static gchar *
-gpm_prefs_format_time_cb (GtkScale *scale,
-			  gdouble   value,
-			  GpmPrefs *prefs)
-{
-	gchar *str;
-	if ((gint) value == NEVER_TIME_ON_SLIDER) {
-		str = g_strdup (_("Never"));
-	} else {
-		str = gpm_get_timestring (value * 60);
-	}
-	return str;
-}
-
-/**
- * gpm_prefs_sleep_slider_changed_cb:
- * @range: The GtkRange object
- * @gpm_pref_key: The GConf key for this preference setting.
- **/
-static void
-gpm_prefs_sleep_slider_changed_cb (GtkRange *range,
-				   GpmPrefs *prefs)
-{
-	int value;
-	char *gpm_pref_key;
-	gboolean sleep_prefix;
-
-	value = (int) gtk_range_get_value (range);
-	sleep_prefix = (gboolean) GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (range), "sleep-prefix"));
-
-	if (value == NEVER_TIME_ON_SLIDER) {
-		/* power manager interprets 0 as Never */
-		value = 0;
-	} else {
-		/* We take away the g-s idle time as the slider represents
-		 * global time but we only do our timeout from when gnome-session
-		 * declares the session idle */
-		if (sleep_prefix)
-			value -= prefs->priv->idle_delay;
-
-		/* policy is in seconds, slider is in minutes */
-		value *= 60;
-	}
-
-	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (range), "conf_key");
-	egg_debug ("Changing %s to %i", gpm_pref_key, value);
-	gconf_client_set_int (prefs->priv->conf, gpm_pref_key, value, NULL);
-}
-
-/**
- * gpm_prefs_setup_sleep_slider:
- * @prefs: This prefs class instance
- * @widget_name: The GtkWidget name
- * @gpm_pref_key: The GConf key for this preference setting.
- **/
-static GtkWidget *
-gpm_prefs_setup_sleep_slider (GpmPrefs *prefs, const gchar *widget_name, const gchar *gpm_pref_key, gboolean sleep_prefix)
-{
-	GtkWidget *widget;
-	gint value;
-	gboolean is_writable;
-	GtkAdjustment *adjustment;
-
-	widget = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder, widget_name));
-	g_signal_connect (G_OBJECT (widget), "format-value",
-			  G_CALLBACK (gpm_prefs_format_time_cb), prefs);
-
-	value = gconf_client_get_int (prefs->priv->conf, gpm_pref_key, NULL);
-	is_writable = gconf_client_key_is_writable (prefs->priv->conf, gpm_pref_key, NULL);
-
-	gtk_widget_set_sensitive (widget, is_writable);
-
-	if (value == 0) {
-		value = NEVER_TIME_ON_SLIDER;
-	} else {
-		/* policy is in seconds, slider is in minutes */
-		value /= 60;
-		if (sleep_prefix)
-			value += prefs->priv->idle_delay;
-	}
-
-	g_object_set_data (G_OBJECT (widget), "sleep-prefix", GUINT_TO_POINTER (sleep_prefix));
-
-	/* set upper */
-	adjustment = gtk_range_get_adjustment (GTK_RANGE (widget));
-	gtk_adjustment_set_upper (adjustment, NEVER_TIME_ON_SLIDER);
-
-	gtk_range_set_value (GTK_RANGE (widget), value);
-
-	g_object_set_data (G_OBJECT (widget), "conf_key", (gpointer) gpm_pref_key);
-
-	g_signal_connect (G_OBJECT (widget), "value-changed",
-			  G_CALLBACK (gpm_prefs_sleep_slider_changed_cb),
-			  prefs);
-
-	return widget;
-}
-
 /**
  * gpm_prefs_brightness_slider_changed_cb:
  * @range: The GtkRange object
@@ -349,16 +220,13 @@ gpm_prefs_brightness_slider_changed_cb (GtkRange *range, GpmPrefs *prefs)
  * @gpm_pref_key: The GConf key for this preference setting.
  **/
 static GtkWidget *
-gpm_prefs_setup_brightness_slider (GpmPrefs    *prefs,
-				   const gchar *widget_name,
-				   const gchar *gpm_pref_key)
+gpm_prefs_setup_brightness_slider (GpmPrefs *prefs, const gchar *widget_name, const gchar *gpm_pref_key)
 {
-	GtkBuilder    *xml = prefs->priv->builder;
 	GtkWidget *widget;
 	int value;
 	gboolean is_writable;
 
-	widget = GTK_WIDGET (gtk_builder_get_object (xml, widget_name));
+	widget = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder, widget_name));
 
 	g_signal_connect (G_OBJECT (widget), "format-value",
 			  G_CALLBACK (gpm_prefs_format_percentage_cb), NULL);
@@ -380,41 +248,47 @@ gpm_prefs_setup_brightness_slider (GpmPrefs    *prefs,
 
 /**
  * gpm_prefs_action_combo_changed_cb:
- * @widget: The GtkWidget object
- * @gpm_pref_key: The GConf key for this preference setting.
  **/
 static void
-gpm_prefs_action_combo_changed_cb (GtkWidget *widget,
-				   GpmPrefs  *prefs)
+gpm_prefs_action_combo_changed_cb (GtkWidget *widget, GpmPrefs *prefs)
 {
-	gchar *value;
+	GpmActionPolicy policy;
+	const GpmActionPolicy *actions;
+	const gchar *gpm_pref_key;
 	const gchar *action;
-	gchar *gpm_pref_key;
+	guint active;
 
-	value = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget));
+	actions = (const GpmActionPolicy *) g_object_get_data (G_OBJECT (widget), "actions");
+	gpm_pref_key = (const gchar *) g_object_get_data (G_OBJECT (widget), "conf_key");
 
-	if (strcmp (value, ACTION_SUSPEND_TEXT) == 0) {
-		action = ACTION_SUSPEND;
-	} else if (strcmp (value, ACTION_HIBERNATE_TEXT) == 0) {
-		action = ACTION_HIBERNATE;
-	} else if (strcmp (value, ACTION_SHUTDOWN_TEXT) == 0) {
-		action = ACTION_SHUTDOWN;
-	} else if (strcmp (value, ACTION_BLANK_TEXT) == 0) {
-		action = ACTION_BLANK;
-	} else if (strcmp (value, ACTION_NOTHING_TEXT) == 0) {
-		action = ACTION_NOTHING;
-	} else if (strcmp (value, ACTION_INTERACTIVE_TEXT) == 0) {
-		action = ACTION_INTERACTIVE;
-	} else {
-		g_assert (FALSE);
-	}
-
-	g_free (value);
-	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (widget), "conf_key");
+	active = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
+	policy = actions[active];
+	action = gpm_action_policy_to_string (policy);
 	egg_debug ("Changing %s to %s", gpm_pref_key, action);
 	gconf_client_set_string (prefs->priv->conf, gpm_pref_key, action, NULL);
 }
 
+/**
+ * gpm_prefs_action_time_changed_cb:
+ **/
+static void
+gpm_prefs_action_time_changed_cb (GtkWidget *widget, GpmPrefs *prefs)
+{
+	guint value;
+	const gint *values;
+	const gchar *gpm_pref_key;
+	guint active;
+
+	values = (const gint *) g_object_get_data (G_OBJECT (widget), "values");
+	gpm_pref_key = (const gchar *) g_object_get_data (G_OBJECT (widget), "conf_key");
+
+	active = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
+	value = values[active];
+
+	egg_debug ("Changing %s to %i", gpm_pref_key, value);
+	gconf_client_set_int (prefs->priv->conf, gpm_pref_key, value, NULL);
+}
+
 /**
  * gpm_prefs_set_combo_simple_text:
  **/
@@ -435,6 +309,15 @@ gpm_prefs_set_combo_simple_text (GtkWidget *combo_box)
 					NULL);
 }
 
+/**
+ * gpm_prefs_actions_destroy_cb:
+ **/
+static void
+gpm_prefs_actions_destroy_cb (GpmActionPolicy *array)
+{
+	g_free (array);
+}
+
 /**
  * gpm_prefs_setup_action_combo:
  * @prefs: This prefs class instance
@@ -443,76 +326,136 @@ gpm_prefs_set_combo_simple_text (GtkWidget *combo_box)
  * @actions: The actions to associate in an array.
  **/
 static void
-gpm_prefs_setup_action_combo (GpmPrefs     *prefs,
-			      const gchar  *widget_name,
-			      const gchar  *gpm_pref_key,
-			      const gchar **actions)
+gpm_prefs_setup_action_combo (GpmPrefs *prefs, const gchar *widget_name,
+			      const gchar *gpm_pref_key, const GpmActionPolicy *actions)
 {
-	GtkBuilder    *xml = prefs->priv->builder;
-	gchar *value;
-	gint i = 0;
+	gchar *value_txt;
+	gint i;
 	gint n_added = 0;
 	gboolean is_writable;
 	GtkWidget *widget;
+	GpmActionPolicy policy;
+	GpmActionPolicy	value;
+	GPtrArray *array;
+	GpmActionPolicy *actions_added;
 
-	widget = GTK_WIDGET (gtk_builder_get_object (xml, widget_name));
+	widget = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder, widget_name));
 	gpm_prefs_set_combo_simple_text (widget);
 
-	value = gconf_client_get_string (prefs->priv->conf, gpm_pref_key, NULL);
+	value_txt = gconf_client_get_string (prefs->priv->conf, gpm_pref_key, NULL);
 	is_writable = gconf_client_key_is_writable (prefs->priv->conf, gpm_pref_key, NULL);
+	value = gpm_action_policy_from_string (value_txt);
 
 	gtk_widget_set_sensitive (widget, is_writable);
 
+	array = g_ptr_array_new ();
 	g_object_set_data (G_OBJECT (widget), "conf_key", (gpointer) gpm_pref_key);
 	g_signal_connect (G_OBJECT (widget), "changed",
 			  G_CALLBACK (gpm_prefs_action_combo_changed_cb), prefs);
 
-	if (value == NULL) {
-		egg_warning ("invalid schema, please re-install");
-		value = g_strdup ("nothing");
+	for (i=0; actions[i] != -1; i++) {
+		policy = actions[i];
+		if (policy == GPM_ACTION_POLICY_SHUTDOWN && !prefs->priv->can_shutdown) {
+			egg_debug ("Cannot add option, as cannot shutdown.");
+		} else if (policy == GPM_ACTION_POLICY_SHUTDOWN && prefs->priv->can_shutdown) {
+			gtk_combo_box_append_text (GTK_COMBO_BOX (widget), _("Shutdown"));
+			g_ptr_array_add (array, GINT_TO_POINTER (policy));
+			n_added++;
+		} else if (policy == GPM_ACTION_POLICY_SUSPEND && !prefs->priv->can_suspend) {
+			egg_debug ("Cannot add option, as cannot suspend.");
+		} else if (policy == GPM_ACTION_POLICY_HIBERNATE && !prefs->priv->can_hibernate) {
+			egg_debug ("Cannot add option, as cannot hibernate.");
+		} else if (policy == GPM_ACTION_POLICY_SUSPEND && prefs->priv->can_suspend) {
+			gtk_combo_box_append_text (GTK_COMBO_BOX (widget), _("Suspend"));
+			g_ptr_array_add (array, GINT_TO_POINTER (policy));
+			n_added++;
+		} else if (policy == GPM_ACTION_POLICY_HIBERNATE && prefs->priv->can_hibernate) {
+			gtk_combo_box_append_text (GTK_COMBO_BOX (widget), _("Hibernate"));
+			g_ptr_array_add (array, GINT_TO_POINTER (policy));
+			n_added++;
+		} else if (policy == GPM_ACTION_POLICY_BLANK) {
+			gtk_combo_box_append_text (GTK_COMBO_BOX (widget), _("Blank screen"));
+			g_ptr_array_add (array, GINT_TO_POINTER (policy));
+			n_added++;
+		} else if (policy == GPM_ACTION_POLICY_INTERACTIVE) {
+			gtk_combo_box_append_text (GTK_COMBO_BOX (widget), _("Ask me"));
+			g_ptr_array_add (array, GINT_TO_POINTER (policy));
+			n_added++;
+		} else if (policy == GPM_ACTION_POLICY_NOTHING) {
+			/* we only add do nothing in the GUI if the user has explicitly specified this in GConf */
+			if (value == GPM_ACTION_POLICY_NOTHING) {
+				gtk_combo_box_append_text (GTK_COMBO_BOX (widget), _("Do nothing"));
+				g_ptr_array_add (array, GINT_TO_POINTER (policy));
+				n_added++;
+			}
+		} else {
+			egg_warning ("Unknown action read from conf: %i", policy);
+		}
 	}
 
-	while (actions[i] != NULL) {
-		if ((strcmp (actions[i], ACTION_SHUTDOWN) == 0) && !prefs->priv->can_shutdown) {
-			egg_debug ("Cannot add option, as cannot shutdown.");
-		} else if (strcmp (actions[i], ACTION_SHUTDOWN) == 0 && prefs->priv->can_shutdown) {
-			gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
-						   ACTION_SHUTDOWN_TEXT);
-			n_added++;
-		} else if ((strcmp (actions[i], ACTION_SUSPEND) == 0) && !prefs->priv->can_suspend) {
-			egg_debug ("Cannot add option, as cannot suspend.");
-		} else if ((strcmp (actions[i], ACTION_HIBERNATE) == 0) && !prefs->priv->can_hibernate) {
-			egg_debug ("Cannot add option, as cannot hibernate.");
-		} else if ((strcmp (actions[i], ACTION_SUSPEND) == 0) && prefs->priv->can_suspend) {
-			gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
-						   ACTION_SUSPEND_TEXT);
-			n_added++;
-		} else if ((strcmp (actions[i], ACTION_HIBERNATE) == 0) && prefs->priv->can_hibernate) {
-			gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
-						   ACTION_HIBERNATE_TEXT);
-			n_added++;
-		} else if (strcmp (actions[i], ACTION_BLANK) == 0) {
-			gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
-						   ACTION_BLANK_TEXT);
-			n_added++;
-		} else if (strcmp (actions[i], ACTION_INTERACTIVE) == 0) {
-			gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
-						   ACTION_INTERACTIVE_TEXT);
-			n_added++;
-		} else if (strcmp (actions[i], ACTION_NOTHING) == 0) {
-			gtk_combo_box_append_text (GTK_COMBO_BOX (widget),
-						   ACTION_NOTHING_TEXT);
-			n_added++;
+	/* save as array _only_ the actions we could add */
+	actions_added = (GpmActionPolicy *) g_ptr_array_free (array, FALSE);
+	actions_added[n_added] = -1;
+	g_object_set_data_full (G_OBJECT (widget), "actions", (gpointer) actions_added, (GDestroyNotify) gpm_prefs_actions_destroy_cb);
+
+	/* set what we have in GConf */
+	for (i=0; actions_added[i] != -1; i++) {
+		policy = actions_added[i];
+		egg_debug ("added: %s", gpm_action_policy_to_string (policy));
+		if (value == policy)
+			 gtk_combo_box_set_active (GTK_COMBO_BOX (widget), i);
+	}
+
+	g_free (value_txt);
+}
+
+/**
+ * gpm_prefs_setup_time_combo:
+ * @prefs: This prefs class instance
+ * @widget_name: The GtkWidget name
+ * @gpm_pref_key: The GConf key for this preference setting.
+ * @actions: The actions to associate in an array.
+ **/
+static void
+gpm_prefs_setup_time_combo (GpmPrefs *prefs, const gchar *widget_name,
+			    const gchar *gpm_pref_key, const gint *values)
+{
+	guint value;
+	gchar *text;
+	guint i;
+	gboolean is_writable;
+	GtkWidget *widget;
+
+	widget = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder, widget_name));
+	gpm_prefs_set_combo_simple_text (widget);
+
+	value = gconf_client_get_int (prefs->priv->conf, gpm_pref_key, NULL);
+	is_writable = gconf_client_key_is_writable (prefs->priv->conf, gpm_pref_key, NULL);
+	gtk_widget_set_sensitive (widget, is_writable);
+
+	g_object_set_data (G_OBJECT (widget), "conf_key", (gpointer) gpm_pref_key);
+	g_object_set_data (G_OBJECT (widget), "values", (gpointer) values);
+
+	/* add each time */
+	for (i=0; values[i] != -1; i++) {
+
+		/* get translation for number of seconds */
+		if (values[i] != 0) {
+			text = gpm_get_timestring (values[i]);
+			gtk_combo_box_append_text (GTK_COMBO_BOX (widget), text);
+			g_free (text);
 		} else {
-			egg_error ("Unknown action read from conf: %s", actions[i]);
+			gtk_combo_box_append_text (GTK_COMBO_BOX (widget), _("Never"));
 		}
 
-		if (strcmp (value, actions[i]) == 0)
-			 gtk_combo_box_set_active (GTK_COMBO_BOX (widget), n_added - 1);
-		i++;
+		/* matches, so set default */
+		if (value == values[i])
+			 gtk_combo_box_set_active (GTK_COMBO_BOX (widget), i);
 	}
 
-	g_free (value);
+	/* connect after set */
+	g_signal_connect (G_OBJECT (widget), "changed",
+			  G_CALLBACK (gpm_prefs_action_time_changed_cb), prefs);
 }
 
 /**
@@ -521,8 +464,7 @@ gpm_prefs_setup_action_combo (GpmPrefs     *prefs,
  * @gpm_pref_key: The GConf key for this preference setting.
  **/
 static void
-gpm_prefs_checkbox_lock_cb (GtkWidget *widget,
-			    GpmPrefs  *prefs)
+gpm_prefs_checkbox_lock_cb (GtkWidget *widget, GpmPrefs *prefs)
 {
 	gboolean checked;
 	gchar *gpm_pref_key;
@@ -543,29 +485,27 @@ gpm_prefs_checkbox_lock_cb (GtkWidget *widget,
  * @gpm_pref_key: The GConf key for this preference setting.
  **/
 static GtkWidget *
-gpm_prefs_setup_checkbox (GpmPrefs    *prefs,
-			  const gchar *widget_name,
-			  const gchar *gpm_pref_key)
+gpm_prefs_setup_checkbox (GpmPrefs *prefs, const gchar *widget_name, const gchar *gpm_pref_key)
 {
-
-	GtkBuilder    *xml = prefs->priv->builder;
 	gboolean checked;
 	GtkWidget *widget;
 
 	egg_debug ("Setting up %s", gpm_pref_key);
 
-	widget = GTK_WIDGET (gtk_builder_get_object (xml, widget_name));
+	widget = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder, widget_name));
 
 	checked = gconf_client_get_bool (prefs->priv->conf, gpm_pref_key, NULL);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), checked);
 
 	g_object_set_data (G_OBJECT (widget), "conf_key", (gpointer) gpm_pref_key);
-	g_signal_connect (widget, "clicked",
-			  G_CALLBACK (gpm_prefs_checkbox_lock_cb), prefs);
 
 	/* manually do the callback in case we hide elements in the cb */
 	gpm_prefs_checkbox_lock_cb (widget, prefs);
 
+	/* setup after set */
+	g_signal_connect (widget, "clicked",
+			  G_CALLBACK (gpm_prefs_checkbox_lock_cb), prefs);
+
 	return widget;
 }
 
@@ -575,8 +515,7 @@ gpm_prefs_setup_checkbox (GpmPrefs    *prefs,
  * @prefs: This prefs class instance
  **/
 static void
-gpm_prefs_close_cb (GtkWidget	*widget,
-		    GpmPrefs	*prefs)
+gpm_prefs_close_cb (GtkWidget *widget, GpmPrefs *prefs)
 {
 	egg_debug ("emitting action-close");
 	g_signal_emit (prefs, signals [ACTION_CLOSE], 0);
@@ -589,36 +528,12 @@ gpm_prefs_close_cb (GtkWidget	*widget,
  * @prefs: This prefs class instance
  **/
 static gboolean
-gpm_prefs_delete_event_cb (GtkWidget *widget,
-			  GdkEvent   *event,
-			  GpmPrefs   *prefs)
+gpm_prefs_delete_event_cb (GtkWidget *widget, GdkEvent *event, GpmPrefs *prefs)
 {
 	gpm_prefs_close_cb (widget, prefs);
 	return FALSE;
 }
 
-/**
- * set_idle_hscale_stops:
- * @prefs: This prefs class instance
- * @widget_name: The widget name
- *
- * Here we make sure that the start of the hscale is set to the
- * gnome-session idle time to avoid confusion.
- **/
-static void
-set_idle_hscale_stops (GpmPrefs    *prefs,
-		       const gchar *widget_name,
-		       gint         gs_idle_time)
-{
-	GtkWidget *widget;
-	widget = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder, widget_name));
-	if (gs_idle_time + 1 > NEVER_TIME_ON_SLIDER) {
-		egg_warning ("gnome-session timeout is really big");
-		return;
-	}
-	gtk_range_set_range (GTK_RANGE (widget), gs_idle_time + 1, NEVER_TIME_ON_SLIDER);
-}
-
 /**
  * gpm_conf_gconf_key_changed_cb:
  *
@@ -636,18 +551,18 @@ gpm_conf_gconf_key_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *e
 	if (value == NULL)
 		return;
 
-	if (strcmp (entry->key, GPM_CONF_BACKLIGHT_BRIGHTNESS_AC) == 0) {
+	if (g_strcmp0 (entry->key, GPM_CONF_BACKLIGHT_BRIGHTNESS_AC) == 0) {
 		widget = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder, "hscale_ac_brightness"));
 		brightness = gconf_value_get_int (value);
 		gtk_range_set_value (GTK_RANGE (widget), brightness);
 	}
 
-	if (strcmp (entry->key, GPM_CONF_DISKS_SPINDOWN_ENABLE_AC) == 0) {
+	if (g_strcmp0 (entry->key, GPM_CONF_DISKS_SPINDOWN_ENABLE_AC) == 0) {
 		widget = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder, "checkbutton_ac_spindown"));
 		enabled = gconf_value_get_bool (value);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), enabled);
 
-	} else if (strcmp (entry->key, GPM_CONF_DISKS_SPINDOWN_ENABLE_BATT) == 0) {
+	} else if (g_strcmp0 (entry->key, GPM_CONF_DISKS_SPINDOWN_ENABLE_BATT) == 0) {
 		widget = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder, "checkbutton_battery_spindown"));
 		enabled = gconf_value_get_bool (value);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), enabled);
@@ -658,17 +573,17 @@ gpm_conf_gconf_key_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *e
 static void
 prefs_setup_notification (GpmPrefs *prefs)
 {
-	gchar	    *icon_policy_str;
-	gint	     icon_policy;
-	GtkWidget   *radiobutton_icon_always;
-	GtkWidget   *radiobutton_icon_present;
-	GtkWidget   *radiobutton_icon_charge;
-	GtkWidget   *radiobutton_icon_critical;
-	GtkWidget   *radiobutton_icon_never;
-	gboolean     is_writable;
+	gchar *icon_policy_str;
+	gint icon_policy;
+	GtkWidget *radiobutton_icon_always;
+	GtkWidget *radiobutton_icon_present;
+	GtkWidget *radiobutton_icon_charge;
+	GtkWidget *radiobutton_icon_low;
+	GtkWidget *radiobutton_icon_never;
+	gboolean is_writable;
 
 	icon_policy_str = gconf_client_get_string (prefs->priv->conf, GPM_CONF_UI_ICON_POLICY, NULL);
-	icon_policy = gpm_tray_icon_mode_from_string (icon_policy_str);
+	icon_policy = gpm_icon_policy_from_string (icon_policy_str);
 	g_free (icon_policy_str);
 
 	radiobutton_icon_always = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder,
@@ -677,8 +592,8 @@ prefs_setup_notification (GpmPrefs *prefs)
 					       "radiobutton_notification_present"));
 	radiobutton_icon_charge = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder,
 					      "radiobutton_notification_charge"));
-	radiobutton_icon_critical = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder,
-					      "radiobutton_notification_critical"));
+	radiobutton_icon_low = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder,
+					   "radiobutton_notification_low"));
 	radiobutton_icon_never = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder,
 					     "radiobutton_notification_never"));
 
@@ -686,7 +601,7 @@ prefs_setup_notification (GpmPrefs *prefs)
 	gtk_widget_set_sensitive (radiobutton_icon_always, is_writable);
 	gtk_widget_set_sensitive (radiobutton_icon_present, is_writable);
 	gtk_widget_set_sensitive (radiobutton_icon_charge, is_writable);
-	gtk_widget_set_sensitive (radiobutton_icon_critical, is_writable);
+	gtk_widget_set_sensitive (radiobutton_icon_low, is_writable);
 	gtk_widget_set_sensitive (radiobutton_icon_never, is_writable);
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_icon_always),
@@ -695,8 +610,8 @@ prefs_setup_notification (GpmPrefs *prefs)
 				      icon_policy == GPM_ICON_POLICY_PRESENT);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_icon_charge),
 				      icon_policy == GPM_ICON_POLICY_CHARGE);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_icon_critical),
-				      icon_policy == GPM_ICON_POLICY_CRITICAL);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_icon_low),
+				      icon_policy == GPM_ICON_POLICY_LOW);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_icon_never),
 				      icon_policy == GPM_ICON_POLICY_NEVER);
 
@@ -706,20 +621,20 @@ prefs_setup_notification (GpmPrefs *prefs)
 			   GINT_TO_POINTER (GPM_ICON_POLICY_PRESENT));
 	g_object_set_data (G_OBJECT (radiobutton_icon_charge), "policy",
 			   GINT_TO_POINTER (GPM_ICON_POLICY_CHARGE));
-	g_object_set_data (G_OBJECT (radiobutton_icon_critical), "policy",
-			   GINT_TO_POINTER (GPM_ICON_POLICY_CRITICAL));
+	g_object_set_data (G_OBJECT (radiobutton_icon_low), "policy",
+			   GINT_TO_POINTER (GPM_ICON_POLICY_LOW));
 	g_object_set_data (G_OBJECT (radiobutton_icon_never), "policy",
 			   GINT_TO_POINTER (GPM_ICON_POLICY_NEVER));
 
 	/* only connect the callbacks after we set the value, else the conf
-	   keys gets written to (for a split second), and the icon flickers. */
+	 * keys gets written to (for a split second), and the icon flickers. */
 	g_signal_connect (radiobutton_icon_always, "clicked",
 			  G_CALLBACK (gpm_prefs_icon_radio_cb), prefs);
 	g_signal_connect (radiobutton_icon_present, "clicked",
 			  G_CALLBACK (gpm_prefs_icon_radio_cb), prefs);
 	g_signal_connect (radiobutton_icon_charge, "clicked",
 			  G_CALLBACK (gpm_prefs_icon_radio_cb), prefs);
-	g_signal_connect (radiobutton_icon_critical, "clicked",
+	g_signal_connect (radiobutton_icon_low, "clicked",
 			  G_CALLBACK (gpm_prefs_icon_radio_cb), prefs);
 	g_signal_connect (radiobutton_icon_never, "clicked",
 			  G_CALLBACK (gpm_prefs_icon_radio_cb), prefs);
@@ -731,7 +646,7 @@ prefs_setup_notification (GpmPrefs *prefs)
 	if (prefs->priv->has_batteries == FALSE) {
 		/* Hide battery radio options if we have no batteries */
 		gtk_widget_hide_all (radiobutton_icon_charge);
-		gtk_widget_hide_all (radiobutton_icon_critical);
+		gtk_widget_hide_all (radiobutton_icon_low);
 	}
 	if (prefs->priv->has_batteries == FALSE && prefs->priv->has_ups == FALSE) {
 		/* Hide battery present option if no ups or primary */
@@ -743,21 +658,41 @@ static void
 prefs_setup_ac (GpmPrefs *prefs)
 {
 	GtkWidget *widget;
-	const gchar  *button_lid_actions[] =
-				{ACTION_NOTHING,
-				 ACTION_BLANK,
-				 ACTION_SUSPEND,
-				 ACTION_HIBERNATE,
-				 ACTION_SHUTDOWN,
-				 NULL};
+	const GpmActionPolicy button_lid_actions[] =
+				{GPM_ACTION_POLICY_NOTHING,
+				 GPM_ACTION_POLICY_BLANK,
+				 GPM_ACTION_POLICY_SUSPEND,
+				 GPM_ACTION_POLICY_HIBERNATE,
+				 GPM_ACTION_POLICY_SHUTDOWN,
+				 -1};
+
+	static const gint computer_times[] =
+		{10*60,
+		 30*60,
+		 1*60*60,
+		 2*60*60,
+		 0, /* never */
+		 -1};
+	static const gint display_times[] =
+		{1*60,
+		 5*60,
+		 10*60,
+		 30*60,
+		 1*60*60,
+		 0, /* never */
+		 -1};
+
+	gpm_prefs_setup_time_combo (prefs, "combobox_ac_computer",
+				    GPM_CONF_TIMEOUT_SLEEP_COMPUTER_AC,
+				    computer_times);
+	gpm_prefs_setup_time_combo (prefs, "combobox_ac_display",
+				    GPM_CONF_TIMEOUT_SLEEP_DISPLAY_AC,
+				    display_times);
 
 	gpm_prefs_setup_action_combo (prefs, "combobox_ac_lid",
 				      GPM_CONF_BUTTON_LID_AC,
 				      button_lid_actions);
-	gpm_prefs_setup_sleep_slider (prefs, "hscale_ac_computer",
-				      GPM_CONF_TIMEOUT_SLEEP_COMPUTER_AC, TRUE);
-	gpm_prefs_setup_sleep_slider (prefs, "hscale_ac_display",
-				      GPM_CONF_TIMEOUT_SLEEP_DISPLAY_AC, FALSE);
+
 	gpm_prefs_setup_brightness_slider (prefs, "hscale_ac_brightness",
 					   GPM_CONF_BACKLIGHT_BRIGHTNESS_AC);
 
@@ -766,8 +701,6 @@ prefs_setup_ac (GpmPrefs *prefs)
 	gpm_prefs_setup_checkbox (prefs, "checkbutton_ac_spindown",
 				  GPM_CONF_DISKS_SPINDOWN_ENABLE_AC);
 
-	set_idle_hscale_stops (prefs, "hscale_ac_computer", prefs->priv->idle_delay);
-
 	if (prefs->priv->has_button_lid == FALSE) {
 		widget = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder, "hbox_ac_lid"));
 		gtk_widget_hide_all (widget);
@@ -787,19 +720,42 @@ prefs_setup_battery (GpmPrefs *prefs)
 	GtkNotebook *notebook;
 	gint page;
 
-	const gchar  *button_lid_actions[] =
-				{ACTION_NOTHING,
-				 ACTION_BLANK,
-				 ACTION_SUSPEND,
-				 ACTION_HIBERNATE,
-				 ACTION_SHUTDOWN,
-				 NULL};
-	const gchar  *battery_critical_actions[] =
-				{ACTION_NOTHING,
-				 ACTION_SUSPEND,
-				 ACTION_HIBERNATE,
-				 ACTION_SHUTDOWN,
-				 NULL};
+	const GpmActionPolicy button_lid_actions[] =
+				{GPM_ACTION_POLICY_NOTHING,
+				 GPM_ACTION_POLICY_BLANK,
+				 GPM_ACTION_POLICY_SUSPEND,
+				 GPM_ACTION_POLICY_HIBERNATE,
+				 GPM_ACTION_POLICY_SHUTDOWN,
+				 -1};
+	const GpmActionPolicy battery_critical_actions[] =
+				{GPM_ACTION_POLICY_NOTHING,
+				 GPM_ACTION_POLICY_SUSPEND,
+				 GPM_ACTION_POLICY_HIBERNATE,
+				 GPM_ACTION_POLICY_SHUTDOWN,
+				 -1};
+
+	static const gint computer_times[] =
+		{10*60,
+		 30*60,
+		 1*60*60,
+		 2*60*60,
+		 0, /* never */
+		 -1};
+	static const gint display_times[] =
+		{1*60,
+		 5*60,
+		 10*60,
+		 30*60,
+		 1*60*60,
+		 0, /* never */
+		 -1};
+
+	gpm_prefs_setup_time_combo (prefs, "combobox_battery_computer",
+				    GPM_CONF_TIMEOUT_SLEEP_COMPUTER_BATT,
+				    computer_times);
+	gpm_prefs_setup_time_combo (prefs, "combobox_battery_display",
+				    GPM_CONF_TIMEOUT_SLEEP_DISPLAY_BATT,
+				    display_times);
 
 	if (prefs->priv->has_batteries == FALSE) {
 		notebook = GTK_NOTEBOOK (gtk_builder_get_object (prefs->priv->builder, "notebook_preferences"));
@@ -815,21 +771,15 @@ prefs_setup_battery (GpmPrefs *prefs)
 	gpm_prefs_setup_action_combo (prefs, "combobox_battery_critical",
 				      GPM_CONF_ACTIONS_CRITICAL_BATT,
 				      battery_critical_actions);
-	gpm_prefs_setup_sleep_slider (prefs, "hscale_battery_computer",
-				      GPM_CONF_TIMEOUT_SLEEP_COMPUTER_BATT, TRUE);
-	gpm_prefs_setup_sleep_slider (prefs, "hscale_battery_display",
-				      GPM_CONF_TIMEOUT_SLEEP_DISPLAY_BATT, FALSE);
 
 	/* set up the battery reduce checkbox */
 	gpm_prefs_setup_checkbox (prefs, "checkbutton_battery_display_reduce",
-	  			  GPM_CONF_BACKLIGHT_BATTERY_REDUCE);
+				  GPM_CONF_BACKLIGHT_BATTERY_REDUCE);
 	gpm_prefs_setup_checkbox (prefs, "checkbutton_battery_display_dim",
 				  GPM_CONF_BACKLIGHT_IDLE_DIM_BATT);
 	gpm_prefs_setup_checkbox (prefs, "checkbutton_battery_spindown",
 				  GPM_CONF_DISKS_SPINDOWN_ENABLE_BATT);
 
-	set_idle_hscale_stops (prefs, "hscale_battery_computer", prefs->priv->idle_delay);
-
 	if (prefs->priv->has_button_lid == FALSE) {
 		widget = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder, "hbox_battery_lid"));
 		gtk_widget_hide_all (widget);
@@ -847,11 +797,34 @@ prefs_setup_ups (GpmPrefs *prefs)
 	GtkNotebook *notebook;
 	gint page;
 
-	const gchar  *ups_low_actions[] =
-				{ACTION_NOTHING,
-				 ACTION_HIBERNATE,
-				 ACTION_SHUTDOWN,
-				 NULL};
+	const GpmActionPolicy ups_low_actions[] =
+				{GPM_ACTION_POLICY_NOTHING,
+				 GPM_ACTION_POLICY_HIBERNATE,
+				 GPM_ACTION_POLICY_SHUTDOWN,
+				 -1};
+
+	static const gint computer_times[] =
+		{10*60,
+		 30*60,
+		 1*60*60,
+		 2*60*60,
+		 0, /* never */
+		 -1};
+	static const gint display_times[] =
+		{1*60,
+		 5*60,
+		 10*60,
+		 30*60,
+		 1*60*60,
+		 0, /* never */
+		 -1};
+
+	gpm_prefs_setup_time_combo (prefs, "combobox_ups_computer",
+				    GPM_CONF_TIMEOUT_SLEEP_COMPUTER_UPS,
+				    computer_times);
+	gpm_prefs_setup_time_combo (prefs, "combobox_ups_display",
+				    GPM_CONF_TIMEOUT_SLEEP_DISPLAY_UPS,
+				    display_times);
 
 	if (prefs->priv->has_ups == FALSE) {
 		notebook = GTK_NOTEBOOK (gtk_builder_get_object (prefs->priv->builder, "notebook_preferences"));
@@ -867,26 +840,23 @@ prefs_setup_ups (GpmPrefs *prefs)
 	gpm_prefs_setup_action_combo (prefs, "combobox_ups_critical",
 				      GPM_CONF_ACTIONS_CRITICAL_UPS,
 				      ups_low_actions);
-	gpm_prefs_setup_sleep_slider (prefs, "hscale_ups_computer",
-				      GPM_CONF_TIMEOUT_SLEEP_COMPUTER_BATT, TRUE);
-	set_idle_hscale_stops (prefs, "hscale_ups_computer", prefs->priv->idle_delay);
 }
 
 static void
 prefs_setup_general (GpmPrefs *prefs)
 {
 	GtkWidget *widget;
-	const gchar  *power_button_actions[] =
-				{ACTION_INTERACTIVE,
-				 ACTION_SUSPEND,
-				 ACTION_HIBERNATE,
-				 ACTION_SHUTDOWN,
-				 NULL};
-	const gchar  *suspend_button_actions[] =
-				{ACTION_NOTHING,
-				 ACTION_SUSPEND,
-				 ACTION_HIBERNATE,
-				 NULL};
+	const GpmActionPolicy power_button_actions[] =
+				{GPM_ACTION_POLICY_INTERACTIVE,
+				 GPM_ACTION_POLICY_SUSPEND,
+				 GPM_ACTION_POLICY_HIBERNATE,
+				 GPM_ACTION_POLICY_SHUTDOWN,
+				 -1};
+	const GpmActionPolicy suspend_button_actions[] =
+				{GPM_ACTION_POLICY_NOTHING,
+				 GPM_ACTION_POLICY_SUSPEND,
+				 GPM_ACTION_POLICY_HIBERNATE,
+				 -1};
 
 	gpm_prefs_setup_action_combo (prefs, "combobox_general_power",
 				      GPM_CONF_BUTTON_POWER,
@@ -901,12 +871,11 @@ prefs_setup_general (GpmPrefs *prefs)
 	}
 }
 
-#ifdef HAVE_GCONF_DEFAULTS
 /**
- * pk_prefs_set_defaults_cb:
+ * gpm_prefs_set_defaults_cb:
  **/
 static void
-pk_prefs_set_defaults_cb (PolKitGnomeAction *default_action, GpmPrefs *prefs)
+gpm_prefs_set_defaults_cb (GtkWidget *widget, GpmPrefs *prefs)
 {
 	GConfClient *client;
 	DBusGProxy *proxy;
@@ -947,27 +916,6 @@ pk_prefs_set_defaults_cb (PolKitGnomeAction *default_action, GpmPrefs *prefs)
 	g_object_unref (proxy);
 }
 
-/**
- * gpk_prefs_setup_policykit:
- *
- * We have to do this before the glade stuff if done as the custom handler needs the actions setup
- **/
-static void
-gpk_prefs_setup_policykit (GpmPrefs *prefs)
-{
-	PolKitAction *pk_action;
-
-	g_return_if_fail (GPM_IS_PREFS (prefs));
-
-	/* set default */
-	pk_action = polkit_action_new ();
-	polkit_action_set_action_id (pk_action, "org.gnome.gconf.defaults.set-system");
-	prefs->priv->default_action = polkit_gnome_action_new_default ("set-defaults", pk_action,
-								       _("Make Default"), NULL);
-	polkit_action_unref (pk_action);
-}
-#endif
-
 /**
  * gpm_prefs_init:
  * @prefs: This prefs class instance
@@ -977,9 +925,6 @@ gpm_prefs_init (GpmPrefs *prefs)
 {
 	GtkWidget *main_window;
 	GtkWidget *widget;
-#ifdef HAVE_GCONF_DEFAULTS
-	GtkWidget *button;
-#endif
 	gint caps;
 	guint retval;
 	GError *error = NULL;
@@ -1022,18 +967,6 @@ gpm_prefs_init (GpmPrefs *prefs)
 
 	main_window = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder, "dialog_preferences"));
 
-#ifdef HAVE_GCONF_DEFAULTS
-	/* we have to do this before we connect up the GtkBuilder file */
-	gpk_prefs_setup_policykit (prefs);
-
-	/* create PolicyKit button */
-	widget = gtk_dialog_get_action_area (GTK_DIALOG (main_window));
-	button = polkit_gnome_action_create_button (prefs->priv->default_action);
-	gtk_box_pack_start (GTK_BOX (widget), button, FALSE, FALSE, 0);
-	gtk_box_reorder_child (GTK_BOX (widget), button, 0);
-	gtk_widget_show (button);
-#endif
-
 	/* Hide window first so that the dialogue resizes itself without redrawing */
 	gtk_widget_hide (main_window);
 	gtk_window_set_default_icon_name (GPM_STOCK_APP_ICON);
@@ -1050,9 +983,12 @@ gpm_prefs_init (GpmPrefs *prefs)
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (gpm_prefs_help_cb), prefs);
 
+	widget = GTK_WIDGET (gtk_builder_get_object (prefs->priv->builder, "button_defaults"));
 #ifdef HAVE_GCONF_DEFAULTS
-	g_signal_connect (prefs->priv->default_action, "activate",
-			  G_CALLBACK (pk_prefs_set_defaults_cb), prefs);
+	g_signal_connect (widget, "clicked",
+			  G_CALLBACK (gpm_prefs_set_defaults_cb), prefs);
+#else
+	gtk_widget_hide (widget);
 #endif
 
 	prefs_setup_ac (prefs);
