@@ -47,7 +47,6 @@
 #include "gpm-engine.h"
 #include "gpm-control.h"
 #include "gpm-common.h"
-#include "gpm-notify.h"
 #include "gpm-stock-icons.h"
 #include "gpm-tray-icon.h"
 
@@ -60,7 +59,6 @@ struct GpmTrayIconPrivate
 {
 	GConfClient		*conf;
 	GpmControl		*control;
-	GpmNotify		*notify;
 	GpmEngine		*engine;
 	GtkStatusIcon		*status_icon;
 	gboolean		 show_suspend;
@@ -140,6 +138,16 @@ gpm_tray_icon_set_tooltip (GpmTrayIcon *icon, const gchar *tooltip)
 	return TRUE;
 }
 
+/**
+ * gpm_tray_icon_get_status_icon:
+ **/
+GtkStatusIcon *
+gpm_tray_icon_get_status_icon (GpmTrayIcon *icon)
+{
+	g_return_val_if_fail (GPM_IS_TRAY_ICON (icon), NULL);
+	return g_object_ref (icon->priv->status_icon);
+}
+
 /**
  * gpm_tray_icon_set_image_from_stock:
  * @filename: The icon name, e.g. GPM_STOCK_APP_ICON, or NULL to remove.
@@ -254,10 +262,12 @@ gpm_tray_icon_suspend_cb (GtkMenuItem *item, gpointer data)
 static void
 gpm_tray_icon_show_statistics_cb (GtkMenuItem *item, gpointer data)
 {
-	const gchar *command = "gnome-power-statistics";
+	gchar *path;
 
-	if (g_spawn_command_line_async (command, NULL) == FALSE)
-		egg_warning ("Couldn't execute command: %s", command);
+	path = g_build_filename (BINDIR, "gnome-power-statistics", NULL);
+	if (g_spawn_command_line_async (path, NULL) == FALSE)
+		egg_warning ("Couldn't execute command: %s", path);
+	g_free (path);
 }
 
 /**
@@ -399,6 +409,8 @@ gpm_tray_icon_popup_menu_cb (GtkStatusIcon *status_icon, guint button, guint32 t
 	GtkMenu *menu = (GtkMenu*) gtk_menu_new ();
 	GtkWidget *item;
 	GtkWidget *image;
+	gchar *path;
+	gboolean ret;
 
 	egg_debug ("icon right clicked");
 
@@ -413,15 +425,22 @@ gpm_tray_icon_popup_menu_cb (GtkStatusIcon *status_icon, guint button, guint32 t
 			  G_CALLBACK (gpm_tray_icon_show_preferences_cb), icon);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-	/* statistics */
-	item = gtk_image_menu_item_new_with_mnemonic (_("Power _History"));
-	image = gtk_image_new_from_icon_name (GPM_STOCK_STATISTICS, GTK_ICON_SIZE_MENU);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-	g_signal_connect (G_OBJECT (item), "activate",
-			  G_CALLBACK (gpm_tray_icon_show_statistics_cb), icon);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	/* this program is optional, and may be in an uninstalled subpackage */
+	path = g_build_filename (BINDIR, "gnome-power-statistics", NULL);
+	ret = g_file_test (path, G_FILE_TEST_EXISTS);
+	g_free (path);
 
-	/* separator for HIG? */
+	/* statistics */
+	if (ret) {
+		item = gtk_image_menu_item_new_with_mnemonic (_("Power _History"));
+		image = gtk_image_new_from_icon_name (GPM_STOCK_STATISTICS, GTK_ICON_SIZE_MENU);
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+		g_signal_connect (G_OBJECT (item), "activate",
+				  G_CALLBACK (gpm_tray_icon_show_statistics_cb), icon);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	}
+
+	/* separator for HIG */
 	item = gtk_separator_menu_item_new ();
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
@@ -613,18 +632,6 @@ gpm_conf_gconf_key_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *e
 	}
 }
 
-/**
- * gpm_tray_status_icon_handler_cb:
- *
- * This attaches any future notifications to the status icon if it exists
- **/
-static gboolean
-gpm_tray_status_icon_handler_cb (GpmTrayIcon *icon)
-{
-	gpm_notify_use_status_icon (icon->priv->notify, icon->priv->status_icon);
-	return FALSE;
-}
-
 /**
  * gpm_tray_icon_init:
  *
@@ -640,9 +647,6 @@ gpm_tray_icon_init (GpmTrayIcon *icon)
 
 	icon->priv->engine = gpm_engine_new ();
 
-	/* use libnotify */
-	icon->priv->notify = gpm_notify_new ();
-
 	/* use the policy object */
 	icon->priv->control = gpm_control_new ();
 
@@ -664,9 +668,6 @@ gpm_tray_icon_init (GpmTrayIcon *icon)
 				 G_CALLBACK (gpm_tray_icon_activate_cb),
 				 icon, 0);
 
-	/* attach the status icon after a few seconds to allow for the session to load properly */
-	g_timeout_add_seconds (GPM_TRAY_ICON_CONNECT_TIMEOUT, (GSourceFunc) gpm_tray_status_icon_handler_cb, icon);
-
 	/* only show the suspend and hibernate icons if we can do the action,
 	   and the policy allows the actions in the menu */
 	allowed_in_menu = gconf_client_get_bool (icon->priv->conf, GPM_CONF_UI_SHOW_ACTIONS_IN_MENU, NULL);
@@ -693,8 +694,6 @@ gpm_tray_icon_finalize (GObject *object)
 
 	tray_icon = GPM_TRAY_ICON (object);
 
-	if (tray_icon->priv->notify != NULL)
-		g_object_unref (tray_icon->priv->notify);
 	g_object_unref (tray_icon->priv->control);
 	g_object_unref (tray_icon->priv->status_icon);
 	g_object_unref (tray_icon->priv->engine);
